/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

dojo.provide("com.ibm.epm.web.core.transport.ServiceRequest");

dojo.declare("com.ibm.epm.web.core.transport.ServiceRequest", null, {

    // -------------------------------------------------------------------------
    // API Methods
    // -------------------------------------------------------------------------
    
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    /**
     * Represents an request for a Jazz service.
     * <p>
     * Example usage:
     * </p>
     * <pre>
     *   var serviceInterface=
     *           "com.ibm.team.workitem.common.service.IWorkItemRestService";
     *   var method= "getWorkitems";
     *   var reqParms= {
     *       ownerName: "Bill Higgins",
     *       isOpen: true,
     *       status: ["new", "assigned"],
     *       priority: 2
     *   };
     *   var ServiceRequest=
     *       new com.ibm.team.repository.web.transport.ServiceRequest(
     *               pathInfo,
     *               method,
     *               reqParms);
     * </pre>
     *
     * @param serviceInterface a URL indicating a particular service available
     *  from a team server.
     * @param method the name of a method following the format getXXX or postXXX
     *  corresponding to a method in the service interface.
     * @param requestParameters an optional object containing properties that
     *  should be sent as request parameters with the request.  Valid properties
     *  include Javascript primitives (string, numeric, boolean) or arrays of
     *  primitives.
     *
     * @throws Exception if serviceInterface is not a string, if method does not
     *  follow the pattern getXXX or postXXX, or if requestParameters is
     *  malformed (i.e. it does not conform to the structure described above).
     *
     *  @since 0.5
     */
    constructor: function(/* object */serviceInterface, /* string */ method, /* string */ requestParams){
        var srClass = com.ibm.epm.web.core.transport.ServiceRequest;
        var srClassName = "com.ibm.epm.web.core.transport.ServiceRequest()";
        
        if (arguments.length < 2 || arguments.length > 3) {
            throw new Error(srClassName + ": invalid number of parameters: " + arguments.length);
        }
        
        // validate input; this will throw an exception if illegal input is discovered
        srClass._validateInput(serviceInterface, method, requestParams);
        
        this._serviceInterface = serviceInterface;
        this._method = method;
        this._params = srClass._makeAllParamsArrays(requestParams);
    },
    
    /**
     * Aborts the service request, causing the response to be ignored and
     * freeing the connection. The response handler will not be called.
     * Has no effect if called on a request that hasn't yet been issued or
     * has already received its response.
     *
     * @since 0.6
     */
    abort: function(){
        if (this._xhr) {
            this._xhr.abort();
        }
    },
    
    /**
     * Returns an array of strings containing the names of the parameters
     * contained in this request. If the request has no parameters, the method
     * returns an empty array.
     *
     * @return an array of strings containing the requests request parameter
     *  names, or an empty array if the request has no parameters.
     *
     * @since 0.5
     */
    getParameterNames: function(){
        var paramNames = [];
        for (var prop in this._params) {
            paramNames.push(prop);
        }
        return paramNames;
    },
    
    /**
     * Returns the extra path info that follows a request's team server service
     * context path (e.g. /jazz/service) but precedes its query string.  Always
     * starts with the '/' character.
     *
     * @since 0.5
     */
    getMethod: function(){
        return this._method;
    },
    
    /**
     * Returns this service request's method type as a string; either "GET" or
     * "POST".
     *
     * @return this service request's method type as a string.
     *
     * @since 0.5
     */
    getMethodType: function(){
        var regex = /^(get|post|put|delete)/;
        var result = this._method.match(regex)[0].toUpperCase();
        return result;
    },
    
    /**
     * Returns an array of strings containing all of the values the given
     * request parameter has, or <code>null</code> if the parameter does not
     * exist.
     *
     * @return an array of strings containing the parameter's values.
     *
     * @since 0.5
     */
    getParameterValues: function(name){
        var srcParamVals = this._params[name];
        if (!srcParamVals) {
            return null;
        }
        var copyParamVals = [];
        for (var i = 0; i < srcParamVals.length; i++) {
            copyParamVals.push(srcParamVals[i]);
        }
        return copyParamVals;
    },
    
    /**
     * Returns the extra path info that follows a request's team server service
     * context path (e.g. /jazz/service) but precedes its query string.  Always
     * starts with the '/' character.
     *
     * @since 0.5
     */
    getServiceInterface: function(){
        return this._serviceInterface;
    },
    
    /**
     * Returns the request's paremeters, represented as a query string, or
     * <code>null</code> if the request does not contain request parameters.
     * Encodes any illegal URL characters with their ASCII code equivalents.
     * <p>
     * TODO: Need to consider providing an optional encoding parameter to
     * support both ASCII and UTF-8 URL encoding (see dojo.io.argsFromMap for an
     * example).
     * </p>
     *
     * @return the request's query string, or <code>null</code> if the request
     *  does not have request parameters.
     *
     *  @since 0.5
     */
    getQueryString: function(){
        return dojo.objectToQuery(this._params);
    },
    
    // ----------------------------------------------------------------------------
    // Internal methods.  *Not* API - do not use!
    // ----------------------------------------------------------------------------
    
    /*
     * (Internal) Called when the request is initiated to set the associated
     * XMLHttpRequest, which is needed to support the abort() feature.
     */
    _setXMLHttpRequest: function(/* XMLHttpRequest */xhr){
        this._xhr = xhr;
    }
});

// ----------------------------------------------------------------------------
// "private" properties and functions.  *Not* API - do not use!
// ----------------------------------------------------------------------------

/*
 * Validates the input to a ServiceRequest constructor.  Throws an exception if
 * any of the input elements is invalid.
 */
com.ibm.epm.web.core.transport.ServiceRequest._validateInput = function(serviceInterface, method, requestParams){
    var srClass = com.ibm.epm.web.core.transport.ServiceRequest;
    var srClassName = "com.ibm.epm.web.core.transport.ServiceRequest()";
    
    if (!srClass._validateServiceInterface(serviceInterface)) {
        throw new Error(srClassName + ": invalid serviceInterface parameter: " + serviceInterface);
    }
    
    if (!srClass._validateMethod(method)) {
        throw new Error(srClassName + ": invalid method parameter: " + method);
    }
    
    if (arguments.length == 3 && !srClass._validateRequestParams(requestParams)) {
        throw new Error(srClassName + ": invalid requestParameters parameter: " + requestParams);
    }
};

/*
 * A serviceInterface is valid if it is a string with more than one character.
 */
com.ibm.epm.web.core.transport.ServiceRequest._validateServiceInterface = function(serviceInterface){
    if (!dojo.isString(serviceInterface)) {
        return false;
    }
    return dojo.trim(serviceInterface).length > 0;
};

/*
 * A method is valid if it is a string that begins with "get" or "post" or "put" or "delete".
 */
com.ibm.epm.web.core.transport.ServiceRequest._validateMethod = function(method){
    if (!dojo.isString(method)) {
        return false;
    }
    var regex = /^(get|post|put|delete)/;
    var result = method.match(regex)[0];
    return result !== null;
};

/*
 * A request parameter object is valid if its properties are all primitive
 * types or arrays of primitive types.
 */
com.ibm.epm.web.core.transport.ServiceRequest._validateRequestParams = function(paramObj){
    for (var prop in paramObj) {
        var propVal = paramObj[prop];
        // if the property value is a simple primitive type, it's fine
        if (this._isPrimitive(propVal)) {
            continue;
        }
        // if the property value is an array, loop through the array and ensure that each element is a primitive
        if (dojo.isArray(propVal)) {
            for (var i = 0; i < propVal.length; i++) {
                if (!this._isPrimitive(propVal[i])) {
                    return false;
                }
            }
        }
        else {
            return false;
        }
    }
    return true;
};

/*
 * Simple test that returns <code>true</code> if aVar is of type string,
 * numeric, or boolean.
 */
com.ibm.epm.web.core.transport.ServiceRequest._isPrimitive = function(aVar){
    return dojo.isString(aVar) || typeof aVar == "number" || typeof aVar == "boolean";
};

/*
 * Converts every primitive property of the internal _params object into a
 * single-element array containing the primitive value.  Must only be called
 * after calling the _validateParameters method.
 */
com.ibm.epm.web.core.transport.ServiceRequest._makeAllParamsArrays = function(paramObj){
    var modifiedParams = {};
    for (var prop in paramObj) {
        var propVal = paramObj[prop];
        if (dojo.isArray(propVal)) {
            modifiedParams[prop] = propVal;
        }
        else {
            modifiedParams[prop] = [propVal];
        }
    }
    return modifiedParams;
};
